unit Telef;

(*
Ttelefon Component for Delphi 2.0 
This is a small freeware component for dialing telephone
nubmers. It is based on two other components :

1.QCCom32 by Scott Pinkham ((c) 1996 Quantum Composers, Bozeman, MT)
2.TDialer by Artchil Gogava ((C) ArGo Software Design, Toronto, Canada)

While I was working with Delphi 1.0 I was using A. Gogava's TDialer component
but since I've moved to Delphi 2.0 I adjusted Gogava's component to win32
enviroment.

Mihaela Mihaljevich
simmiha@filozof.ffzg.hr
Zagreb, Croatia

Thanks to:
     Scott Pinkham ((c) 1996 Quantum Composers, Bozeman, MT)
     Artchil Gogava ((C) ArGo Software Design, Toronto, Canada)

USAGE:
In order to dial a telephone number you just assign it to TelNumber
and then call procedure Dial (e.g. Telefon1.TelNumber:='123456';
                                  Telefon1.Dial;)
If you need your modem to recognize DIALTONE, put 'ATX2' as your
InitStr
Happy programming!
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type

  TCPort = (COM1,COM2,COM3,COM4);
  TDMethod  = (dmTone,dmPulse);

  TTelefon = class(TComponent)
  private
    { Private declarations }
    hPort: LongInt; 
    FCPort : TCPort;
    FTelNumber : string;
    FDMethod : TDMethod;
    FInitStr : string;
  protected
    { Protected declarations }
    function Open: Boolean;
    function Write(const Podaci: String): Boolean;
    procedure Close;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Dial;

    published
    { Published declarations }
     property CPort : TCPort read FCPort
                 write FCPort;
     property InitStr : string read FInitStr
                 write FInitStr;
     property DMethod  : TDMethod read FDMethod
                 write FDMethod;
     property TelNumber : string read FTelNumber
                 write FTelNumber;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TTelefon]);
end;

constructor TTelefon.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	{ set default property values }
	hPort := INVALID_HANDLE_VALUE; { default value is invalid}
        FCPort:=COM2;
        FDMethod:=dmTone;
        FInitStr:='';
        FTelNumber:='1234';
end;

destructor TTelefon.Destroy;
begin
   inherited Destroy;	{ destroy ancestor class }
end;

{ Close the COM port }
procedure TTelefon.Close;
begin
	if hPort <> INVALID_HANDLE_VALUE then CloseHandle(hPort);
	hPort := INVALID_HANDLE_VALUE;
end;

function TTelefon.Open:Boolean;
   var
      Port,Poruka:string;
      Prekini:Boolean;
   begin
      Prekini:=False;
      Port:='';
      Poruka:='';
      case FCPort of
         COM1:   Port:='COM1';
         COM2:   Port:='COM2';
         COM3:   Port:='COM3';
         COM4:   Port:='COM4';
      end;
      if hPort<>INVALID_HANDLE_VALUE then Close;

      repeat
      begin
         hPort:=CreateFile(PChar(Port),GENERIC_READ or GENERIC_WRITE,0,nil,
                           OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,LongInt(0));

         if hPort=INVALID_HANDLE_VALUE then begin
            Poruka:='Error opening ' + Port;
            if MessageDlg(Poruka,mtWarning,[mbAbort,mbRetry],0)=idAbort then
               Prekini:=True
            else
               Prekini:=False;
            end;
         end;
      until (hPort<>INVALID_HANDLE_VALUE) or (Prekini=True);
      Result:=(hPort<>INVALID_HANDLE_VALUE);
   end;

function TTelefon.Write(const Podaci: String): Boolean;
var
	Zapisani: DWord;
 begin
      	Zapisani := 0;
	Result := False; { False is default }

	if hPort <> INVALID_HANDLE_VALUE then
	begin
		WriteFile(hPort, PChar(Podaci)^, Length(Podaci), Zapisani, nil);
		if Zapisani = Length(Podaci) then Result := True;
	end;
end;


procedure TTelefon.Dial;
   var
      s:string;
begin
   //Openning port
   if Open=False then begin  //operation failed
      MessageDlg('Not able to open port ',mtError,[mbOk], 0);
      Exit;
   end;

   //InitString is being sent, ig there is one
   if FInitStr<>'' then begin
      s:=FInitStr+^M^J;
      if Write(s)=False then begin //Operation failed
         MessageDlg('Error initializing modem ',mtError,[mbOk], 0);
         Close;//Closing port and exiting
         Exit;
      end;
   end;

   //Another chance to change our mind
   if MessageDlg('About to dial: '+FTelNumber+'. Are you sure?',
      mtConfirmation, [mbYes,mbNo], 0)<>mrYes then begin
        Close;
        Exit;
   end;

  //Creating dial string
  s:=Concat('ATDP',FTelNumber,^M^J);
  if FDMethod=dmTone then s[4]:='T';

  //Writing to a port
  if Write(s)=False then begin
    MessageDlg('Error dialing number ',mtError,[mbOk], 0);
    Close;
    //Closing port and exiting
    Exit;
  end
  else begin //If everythin so far is OK
     if Application.MessageBox('Pick up the receiver','Dialing...'
       ,MB_TASKMODAL+MB_OK)= IDOK then
       Write('ATH'^M^J);
  end;
end;
  
end.
 